"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es6.regexp.to-string");

var _undefined = _interopRequireDefault(require("@polkadot/util/is/undefined"));

var _logger = _interopRequireDefault(require("@polkadot/util/logger"));

var _toHex = _interopRequireDefault(require("@polkadot/util/u8a/toHex"));

// Copyright 2017-2018 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
const l = (0, _logger.default)('db/overlay');

class OverlayDb {
  constructor(wrapped) {
    this.cache = void 0;
    this.wrapped = void 0;
    this.cache = {};
    this.wrapped = wrapped;
  }

  checkpoint() {
    l.debug(() => ['checkpoint at', (0, _toHex.default)(this.getRoot())]);
    this.wrapped.checkpoint();
  }

  commit() {
    this.wrapped.commit();
    l.debug(() => ['committed at', (0, _toHex.default)(this.getRoot())]);
  }

  revert() {
    this.cache = {};
    this.wrapped.revert();
    l.debug(() => ['reverted to', (0, _toHex.default)(this.getRoot())]);
  }

  del(key) {
    this.cache[key.toString()] = null;
    return this.wrapped.del(key);
  }

  get(key) {
    const keyStr = key.toString();

    if (!(0, _undefined.default)(this.cache[keyStr])) {
      return this.cache[keyStr];
    }

    const result = this.wrapped.get(key);
    this.cache[keyStr] = result;
    return result;
  }

  put(key, value) {
    this.cache[key.toString()] = value.slice();
    return this.wrapped.put(key, value);
  }

  getRoot() {
    return this.wrapped.getRoot();
  }

  setRoot(value) {
    return this.wrapped.setRoot(value);
  }

  terminate() {
    return this.wrapped.terminate();
  }

}

exports.default = OverlayDb;